package cn.hanghui.outapi.platform.common.utils;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.binary.Hex;
import org.bouncycastle.asn1.gm.GMNamedCurves;
import org.bouncycastle.asn1.gm.GMObjectIdentifiers;
import org.bouncycastle.asn1.x9.X9ECParameters;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPrivateKey;
import org.bouncycastle.jcajce.provider.asymmetric.ec.BCECPublicKey;
import org.bouncycastle.jcajce.provider.asymmetric.util.EC5Util;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.jce.spec.ECNamedCurveSpec;
import org.bouncycastle.math.ec.ECCurve;
import java.util.Base64;

import java.math.BigInteger;
import java.security.*;
import java.security.spec.*;

/**
 * @author Ivan
 * @description
 * @date 2020/5/27
 */
public class SM2KeyUtil {
    public static final BouncyCastleProvider bcProvider = new BouncyCastleProvider();

    private static final X9ECParameters ecParameters;

    private static final ECNamedCurveSpec sm2Spec;

    private static final KeyPairGenerator keyGenerator;
    private static final KeyFactory keyFactory;
    private static final ECCurve curve;


    static {
        try {
            // 获取SM2 椭圆曲线推荐参数
            ecParameters = GMNamedCurves.getByName("sm2p256v1");

            // 构造EC 算法参数
            sm2Spec = new ECNamedCurveSpec(
                    // 设置SM2 算法的 OID
                    GMObjectIdentifiers.sm2p256v1.toString()
                    // 设置曲线方程
                    , ecParameters.getCurve()
                    // 椭圆曲线G点
                    , ecParameters.getG()
                    // 大整数N
                    , ecParameters.getN());

            // bouncycastle算法提供器


            keyGenerator = KeyPairGenerator.getInstance("EC", bcProvider);
            keyGenerator.initialize(sm2Spec, new SecureRandom());

            keyFactory = KeyFactory.getInstance("EC", bcProvider);
            curve = ecParameters.getCurve();
        } catch (NoSuchAlgorithmException | InvalidAlgorithmParameterException e) {
            throw new RuntimeException("无法提供SM2算法", e);
        }
    }

    public static KeyPair genKeyPair() {
        return keyGenerator.generateKeyPair();
    }

    public static byte[] encodeWithPoint(BCECPublicKey pub) {
        return pub.getQ().getEncoded(false);
    }

    public static BCECPublicKey decodeWithPoint(byte[] data) throws InvalidKeySpecException {
        ECPoint point = EC5Util.convertPoint(curve.decodePoint(data));
        return (BCECPublicKey) keyFactory.generatePublic(new ECPublicKeySpec(point, sm2Spec));
    }

    public static String encodeStrWithPoint(BCECPublicKey pub) {
        return Hex.encodeHexString(encodeWithPoint(pub),false);
    }

    public static BCECPublicKey decodeWithPoint(String hex) throws InvalidKeySpecException, DecoderException {
        return decodeWithPoint(Hex.decodeHex(hex));
    }

    public static String getX64(BCECPublicKey pub) {
        return Base64.getEncoder().encodeToString(pub.getQ().getXCoord().getEncoded());
    }

    public static String getY64(BCECPublicKey pub) {
        return Base64.getEncoder().encodeToString(pub.getQ().getYCoord().getEncoded());
    }

    public static BCECPublicKey fromXY(String x64, String y64) throws InvalidKeySpecException {
        return fromXY(Base64.getDecoder().decode(x64), Base64.getDecoder().decode(y64));
    }

    public static byte[] getX(BCECPublicKey pub) {
        return pub.getQ().getXCoord().getEncoded();
    }

    public static byte[] getY(BCECPublicKey pub) {
        return pub.getQ().getYCoord().getEncoded();
    }

    public static BCECPublicKey fromXY(byte[] x, byte[] y) throws InvalidKeySpecException {
        ECPoint point = EC5Util.convertPoint(curve.createPoint(new BigInteger(1, x), new BigInteger(1, y)));
        return (BCECPublicKey) keyFactory.generatePublic(new ECPublicKeySpec(point, sm2Spec));
    }

    public static byte[] encodeWithBigInteger(BCECPrivateKey pri) {
        return pri.getD().toByteArray();
    }

    public static BCECPrivateKey decodeWithBigInteger(byte[] data) throws InvalidKeySpecException {
        return (BCECPrivateKey) keyFactory.generatePrivate(new ECPrivateKeySpec(new BigInteger(1, data), sm2Spec));
    }

    public static String encodeStrWithBigInteger(BCECPrivateKey pri) {
        return Hex.encodeHexString(encodeWithBigInteger(pri),false);
    }

    public static BCECPrivateKey decodeWithBigInteger(String hex) throws InvalidKeySpecException, DecoderException {
        return decodeWithBigInteger(Hex.decodeHex(hex));
    }

    public static byte[] encodeWithKey(Key key) {
        return key.getEncoded();
    }

    public static BCECPublicKey decodeWithX509(byte[] data) throws InvalidKeySpecException {
        return (BCECPublicKey) keyFactory.generatePublic(new X509EncodedKeySpec(data));
    }

    public static BCECPrivateKey decodeWithPKCS8(byte[] data) throws InvalidKeySpecException {
        return (BCECPrivateKey) keyFactory.generatePrivate(new PKCS8EncodedKeySpec(data));
    }

    public static String encodeStrWithKey(Key key) {
        return Base64.getEncoder().encodeToString(encodeWithKey(key));
    }

    public static BCECPublicKey decodeWithX509(String base64) throws InvalidKeySpecException {
        return decodeWithX509(Base64.getDecoder().decode(base64.replaceAll("-----.*-----", "")));
    }

    public static BCECPrivateKey decodeWithPKCS8(String base64) throws InvalidKeySpecException {
        return decodeWithPKCS8(Base64.getDecoder().decode(base64.replaceAll("-----.*-----", "")));
    }

}
